SystemOrganization addCategory: #'Arduino-EtoysCompiler'!

!AttachableDeviceMorph methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 8/15/2011 12:58'!
detectMainMorph
	"Looks for the arduino board in which it is connected and returns it"
	World allMorphsDo: [:morph |
		((morph isKindOf: ArduinoMorph) and: [self isAttachedTo: morph])
			ifTrue: [^ morph]
		].
	^ self error: 'The device is not properly attached to any arduino'! !

!AttachableDeviceMorph methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 8/15/2011 10:21'!
download: anArduinoCTranslatedProgram
	^ self detectMainMorph download: anArduinoCTranslatedProgram! !

!AttachableDeviceMorph methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 8/15/2011 16:20'!
isAttachedTo: anArduinoMorph 
	^ device isAttachedToArduino: anArduinoMorph arduino! !

!AttachableDeviceMorph methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 8/15/2011 10:21'!
playerTranslator
	^ self detectMainMorph playerTranslator! !

!ArduinoSketch methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 12/27/2011 17:53'!
getBooleanValue: pinNumber
	<returnType: #(CBoolean)>
	<argTypes: #(CLong)>
	<c: 'getBooleanValue({1})'>! !

!ArduinoSketch methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 12/27/2011 17:52'!
getMode: pinNumber
	<returnType: #(CByte)>
	<argTypes: #(CLong)>
	<c: 'getMode({1})'>! !

!ArduinoSketch methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 12/27/2011 17:53'!
getValue: pinNumber
	<returnType: #(CLong)>
	<argTypes: #(CLong)>
	<c: 'getValue({1})'>! !

!ArduinoSketch methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 12/27/2011 17:54'!
setBooleanValue: pinNumber value: value
	<argTypes: #(CLong CBoolean)>
	<c: 'setBooleanValue({1}, {2})'>! !

!ArduinoSketch methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 12/27/2011 17:53'!
setMode: pinNumber value: value
	<argTypes: #(CLong CByte)>
	<c: 'setMode({1}, {2})'>! !

!ArduinoSketch methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 12/27/2011 17:54'!
setValue: pinNumber value: value
	<argTypes: #(CLong CLong)>
	<c: 'setValue({1}, {2})'>! !

RBProgramNodeVisitor subclass: #ArduinoPlayerTranslator
	instanceVariableNames: 'mainMorph currentMorph methods digitalPins analogPins variables getterSelectors setterSelectors scriptSelectors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-EtoysCompiler'!

!ArduinoPlayerTranslator class methodsFor: 'accessing' stamp: 'Richo 8/16/2011 13:06'!
allPrimitiveRulesSelectors
	^ Array streamContents: [:stream |
		"Collect all primitives in the ArduinoSketch protocol..."
		(Pragma allNamed: #c: from: ArduinoSketch to: CObject)
			do: [:pragma | stream nextPut: pragma method selector].
		"... and in all the primitive types' protocol as well"
		CPrimitiveType withAllSubclassesDo: [:each |
			(Pragma allNamed: #c: in: each)
				do: [:pragma | stream nextPut: pragma method selector]]]! !

!ArduinoPlayerTranslator class methodsFor: 'instance creation' stamp: 'Richo 8/11/2011 17:21'!
on: anArduinoMorph 
	^ self basicNew
			initializeWith: anArduinoMorph;
			yourself! !

!ArduinoPlayerTranslator methodsFor: 'visiting' stamp: 'Richo 8/16/2011 13:04'!
acceptMessageNode: aMessageNode 
	| selector |
	selector := aMessageNode selector.
	"Check for invalid selector or invalid receiver"
	self assert: [self isValidSelector: selector];		
		assert: [self isValidReceiver: aMessageNode receiver].
	"Is it a rule?"
	(self rules includesKey: selector)
		ifTrue: [^(self rules at: selector)
				value: aMessageNode].
	"Is it an etoys message?"
	(self etoysMessages includesKey: selector)
		ifTrue: [(self etoysMessages at: selector)
				value: aMessageNode].
	"Is it a getter?"
	(self allGetterSelectors includes: selector)
		ifTrue: [| newNode | 
			self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: [newNode := self variable: (self slotForGetter: selector)].
			aMessageNode
				become: (newNode parent: aMessageNode parent).
			^ self visitNode: aMessageNode].
	"Is it a setter?"
	(self allSetterSelectors includes: selector)
		ifTrue: [| newNode | 
			self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: [newNode := RBAssignmentNode
								variable: (self variable: (self slotForSetter: selector))
								value: aMessageNode arguments first].
			aMessageNode
				become: (newNode parent: aMessageNode parent).
			^ self visitNode: aMessageNode].
	"Is it a script call?"
	(self allScriptSelectors includes: selector)
		ifTrue: [self
				withCurrentMorph: (self morphFor: aMessageNode receiver name)
				do: ["Check for recursive call"
					self assert: [self isNotRecursiveCall: aMessageNode].
					aMessageNode
						selector: (self asLegalName: aMessageNode selector);
						
						receiver: (self variable: 'self')]].
	"We still need to visit the receiver and the arguments"
	^ super acceptMessageNode: aMessageNode! !

!ArduinoPlayerTranslator methodsFor: 'visiting' stamp: 'Richo 8/16/2011 13:03'!
acceptMethodNode: aMethodNode 
	"It goes over the method's arguments adding the #argTypes: pragma if necessary and modifies the selector to have a legal name"
	aMethodNode allArgumentVariables size = 1 
		ifTrue: [| varType | 
			varType := self asArduinoType: (self currentPlayerClass scripts at: aMethodNode selector) argumentVariables first variableType.
			aMethodNode pragmas: (RBParser parseMethod: 'm <argTypes: #(' , varType , ')>') pragmas].
	aMethodNode
		selector: (self asLegalName: aMethodNode selector).
	^ super acceptMethodNode: aMethodNode! !

!ArduinoPlayerTranslator methodsFor: 'visiting' stamp: 'Richo 8/16/2011 13:03'!
acceptSequenceNode: aSequenceNode
	"If one of the statements has an invalid tile it will ignore it entirely. Only assertion failures will be ignored, other errors will be signaled, though"
	aSequenceNode statements do: [:each | [self visitNode: each] on: AssertionFailure do: [self becomeNoOp: each]]! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/16/2011 13:07'!
allGetterSelectors
	^ getterSelectors inject: #() into: [:subTotal :next | subTotal, next]! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/15/2011 16:35'!
allMorphs
	"Returns a collection with all the morphs composing the robot (it could contain nils)"
	^ {mainMorph}, 
		digitalPins values flattened, 
		analogPins values flattened! !

!ArduinoPlayerTranslator methodsFor: 'iterating' stamp: 'Richo 8/15/2011 16:24'!
allMorphsDo: aBlock
	"Iterates over all morphs composing the robot taking care of posible nil occurrences"
	self allMorphs do: [:each |
		each notNil ifTrue: [aBlock value: each]
	]! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/16/2011 13:08'!
allScriptSelectors
	^ scriptSelectors inject: #() into: [:subTotal :next | subTotal, next]! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/16/2011 13:08'!
allSetterSelectors
	^ setterSelectors inject: #() into: [:subTotal :next | subTotal, next]! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 10/24/2011 13:53'!
arduinoIndexOf: aVariableNode
	^ [self arduinoIndexOf: aVariableNode in: digitalPins]
		on: Error
		do: [:ign |
			14 + (self arduinoIndexOf: aVariableNode in: analogPins)]! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 10/21/2011 20:38'!
arduinoIndexOf: aVariableNode in: aCollection
	"Basically, it tries to determine if the variable node is representing an object in the collection. If it is, it returns its index"
	| index findIndex |
	findIndex := [:morph || ret |
		ret := nil.
		aCollection keysAndValuesDo: [:key :value |
			(value includes: morph)
				ifTrue: [ret := key]
			].
		ret].
	aVariableNode name = 'self' 
		ifTrue: [index := findIndex value: currentMorph]
		ifFalse: [index := findIndex value: (self morphFor: aVariableNode name)].
	^ index isNil ifTrue: [self error: 'Index not found'] ifFalse: [index].
! !

!ArduinoPlayerTranslator methodsFor: 'converting' stamp: 'Richo 8/16/2011 12:57'!
asArduinoType: aSymbol
	"If the type is not valid just return the symbol. Hack to allow #firstTick to have #u32 type"
	^ aSymbol caseOf: {
		[#Number] -> [#CLong].
		[#Boolean] -> [#CBoolean].
		[#String] -> [#CString]
		} otherwise: [aSymbol]! !

!ArduinoPlayerTranslator methodsFor: 'converting' stamp: 'Richo 8/16/2011 12:22'!
asLegalName: aString
"All variables, method names and task names must begin with a prefix representing the receiver."
^ currentMorph assuredPlayer uniqueNameForReference asLegalSelector, aString capitalized! !

!ArduinoPlayerTranslator methodsFor: 'converting' stamp: 'Richo 8/16/2011 12:21'!
asLegalScriptFlag: aSymbol 
	"Returns the name of the flag variable used to stop this script"
	^ self asLegalName: (self asScriptFlag: aSymbol)! !

!ArduinoPlayerTranslator methodsFor: 'converting' stamp: 'Richo 8/16/2011 12:22'!
asLegalScriptLastTime: aByteSymbol 
	^ self asLegalName: (self asScriptLastTime: aByteSymbol)! !

!ArduinoPlayerTranslator methodsFor: 'converting' stamp: 'Richo 8/16/2011 12:21'!
asScriptFlag: aSymbol
	"Returns the name of the flag variable used to stop this script"
	^ (self asScriptName: aSymbol), 'Stepping'! !

!ArduinoPlayerTranslator methodsFor: 'converting' stamp: 'Richo 8/16/2011 12:23'!
asScriptLastTime: aSymbol
	^ aSymbol , 'LastTime'! !

!ArduinoPlayerTranslator methodsFor: 'converting' stamp: 'Richo 8/16/2011 12:21'!
asScriptName: aSymbol
	"Returns the name of this script"
	^ aSymbol, 'Script'! !

!ArduinoPlayerTranslator methodsFor: 'converting' stamp: 'Richo 8/16/2011 13:09'!
becomeNoOp: aParseNode
	"Transforms the argument in an empty statement:
	
	self c: ''.
	"
	| noOp |
	noOp := RBMessageNode receiver: (self variable: 'self') selector: #c: arguments: {self literal: ''}.
	aParseNode become: (noOp parent: aParseNode parent)! !

!ArduinoPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 8/15/2011 16:23'!
collectDevices
	"Finds out which device morphs are connected to the arduino in which pins (digital or analog).
	It's not the cleanest way of doing it but it works..."
	| arduino |
	arduino := mainMorph arduino.
	digitalPins := ((2 to: 13)
		collect: [:n | n -> OrderedCollection new]) as: Dictionary.
	analogPins := ((0 to: 5)
		collect: [:n | n -> OrderedCollection new]) as: Dictionary.
	World allMorphsDo: [:morph |
		((morph isKindOf: AttachableDeviceMorph)
		and: [morph isAttachedTo: mainMorph])
			ifTrue: [
				arduino digitalPins do: [:pin |
					(morph device isAttachedToPin: pin)
						ifTrue: [(digitalPins at: pin number) add: morph]
				].
				arduino analogPins do: [:pin |
					(morph device isAttachedToPin: pin)
						ifTrue: [(analogPins at: pin number) add: morph]
				]
		]
	]! !

!ArduinoPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 10/21/2011 16:30'!
collectGetterSelectors
	"Collects all selectors corresponding to a slot getter in a dictionary whose keys are the morphs owning the slot"
	getterSelectors := Dictionary new.
	self allMorphsDo: [:morph | | selectors |
		selectors := getterSelectors at: morph ifAbsentPut: [OrderedCollection new].
		selectors addAll: (morph assuredPlayer slotInfo keys collect: [:selector | Utilities getterSelectorFor: selector])].
! !

!ArduinoPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 8/15/2011 13:04'!
collectRelevantInformation
	self collectDevices; 
		 collectSlots;
		 collectGetterSelectors;
		 collectSetterSelectors;
		 collectScriptSelectors! !

!ArduinoPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 10/21/2011 16:30'!
collectScriptSelectors
	"Collects all selectors corresponding to a script in a dictionary whose keys are the morphs owning the methods"
	scriptSelectors := Dictionary new.
	self
		allMorphsDo: [:morph | 
			| selectors | 
			selectors := scriptSelectors
						at: morph
						ifAbsentPut: [OrderedCollection new].
			selectors
				addAll: (morph assuredPlayer class selectors
						reject: [:selector | ((getterSelectors at: morph)
									includes: selector)
								or: [(setterSelectors at: morph)
										includes: selector]])] ! !

!ArduinoPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 10/21/2011 16:31'!
collectSetterSelectors
	"Collects all selectors corresponding to a slot setter in a dictionary whose keys are the morphs owning the slot"
	setterSelectors := Dictionary new.
	self allMorphsDo: [:morph | | selectors |
		selectors := setterSelectors at: morph ifAbsentPut: [OrderedCollection new].
		selectors addAll: (morph assuredPlayer slotInfo keys collect: [:selector | Utilities setterSelectorFor: selector])].
! !

!ArduinoPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 10/21/2011 16:32'!
collectSlots
	"Adds all the slots and they respective types to the variables dictionary."
	variables := Dictionary new.
	self allMorphsDo: [:morph | | slots |
		slots := variables at: morph ifAbsentPut: [Dictionary new].
		morph assuredPlayer slotInfo 
			keysAndValuesDo: [:slotName :slotInfo | 
				slots at: slotName put: (slotInfo type -> (morph assuredPlayer instVarNamed: slotName))]]! !

!ArduinoPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 12/29/2011 12:37'!
commonDevicesRules
| selfVar booleanGetter booleanSetter getter setter |
selfVar := self variable: 'self'.
booleanGetter := [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		node arguments: {self literal: (self arduinoIndexOf: node receiver)};
			selector: #getBooleanValue:;
			receiver: selfVar].
	super acceptMessageNode: node].
booleanSetter := [:node | 
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		node arguments: {self literal: (self arduinoIndexOf: node receiver)},
							node arguments;
			selector: #setBooleanValue:value:;
			receiver: selfVar].
	super acceptMessageNode: node].
getter := [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		node arguments: {self literal: (self arduinoIndexOf: node receiver)};
			selector: #getValue:;
			receiver: selfVar].
	super acceptMessageNode: node].
setter := [:node | 
	self withCurrentMorph: (self morphFor: node receiver name) do: [
		node arguments: {self literal: (self arduinoIndexOf: node receiver)},
							node arguments;
			selector: #setValue:value:;
			receiver: selfVar].
	super acceptMessageNode: node].
^ {
	"LED"
	#getIsLedOn -> booleanGetter.
	#setIsLedOn: -> booleanSetter.
	"Pushbutton"
	#getIsPressed -> booleanGetter.
	"Switch"
	#getIsOn -> booleanGetter.
	"Tilt switch"
	#getTiltValue -> getter.
	"Photoresistor/Optocoupler"
	#getLightValue -> getter.
	"Potentiometer"
	#getPotentiometerValue -> getter.
	"Thermistor"
	#getTemperatureValue -> getter.
	"Pwm led"
	#setLedValue: -> setter.
	#getLedValue -> getter.
	"Buzzer"
	#setBuzzerValue:  -> setter.
	#getBuzzerValue -> getter.
	"Relay"
	#getIsRelayOpen -> booleanGetter.
	#setIsRelayOpen: -> booleanSetter.
}! !

!ArduinoPlayerTranslator methodsFor: 'translating' stamp: 'Richo 8/16/2011 12:20'!
compileLoopMethod
	methods add: (String streamContents: [:stream |
		stream nextPutAll: 'loop
	| now |
	<tempTypes: #(CUnsignedLong)>
	now := self millis.
	'.
	self scriptSelectorsDo: [:selector || code |
		code := '({1} and: [now - {2} > {3}])
		ifTrue: [{4} timesRepeat: [self {5}].
			{2} := now].' 
			format: {
				(self asLegalScriptFlag: selector).
				(self asLegalScriptLastTime: selector).
				(self delayOfScript: selector).
				(self frequencyOfScript: selector).
				(self asLegalName: selector)}.
		stream nextPutAll: code.
		]
	])! !

!ArduinoPlayerTranslator methodsFor: 'translating' stamp: 'Richo 12/27/2011 18:15'!
compileSetUpMethod
| methodSource |
methodSource := WriteStream on: ''.
methodSource nextPutAll: 'setup'; cr.
"Configure sensors"
digitalPins keysAndValuesDo: [:key :value |
	value notEmpty
		ifTrue: [| mode |
		mode := (mainMorph arduino digitalPin: key) mode.
		methodSource
			nextPutAll: 'self setMode: ';
			nextPutAll: key asString;
			nextPutAll: 'value:';
			space;
			nextPutAll: (true caseOf: {
				[mode isPwm] -> ['2'].
				[mode isOutput] -> ['1'].
				[mode isInput] -> ['0']
				});
			nextPutAll: '.';
			cr.
	]
].
analogPins keysAndValuesDo: [:key :value |
	value notEmpty
		ifTrue: [| mode |
		mode := (mainMorph arduino analogPin: key) mode.
		methodSource
			nextPutAll: 'self setMode:';
			nextPutAll: (key + 14) asString;
			nextPutAll: 'value:';
			space;
			nextPutAll: (true caseOf: {
				[mode isPwm] -> ['2'].
				[mode isOutput] -> ['1'].
				[mode isInput] -> ['0']
				});
			nextPutAll: '.';
			cr.
	]
].


"Initialize variables"
self variablesDo: [:varName :varTypeAndValue | | value |
	value := varTypeAndValue value.
	value := (value isKindOf: RBProgramNode) 
				ifTrue: [value formattedCode]  "Is a message send"
				ifFalse: [value printString]. "Is a literal"
	methodSource 
		nextPutAll: (self asLegalName: varName), ' := ' , value , '.';
		cr].
methods add: methodSource contents! !

!ArduinoPlayerTranslator methodsFor: 'compiling' stamp: 'Richo 8/16/2011 12:27'!
compileTempProgramSubclass
	| instVarNames instVarTypes |
	instVarNames := ''.
	instVarTypes := #().
	"First collect all variables"
	self
		variablesDo: [:varName :varTypeAndValue | 
			instVarNames := instVarNames, (self asLegalName: varName) , ' '.
			instVarTypes := instVarTypes, {self asArduinoType: varTypeAndValue key}].
	"Then create the class"
	self createTempProgramSubclassInstVarNames: instVarNames
		 instVarTypes: instVarTypes.
	"Finally compile all methods"
	methods
		do: [:each | self programClass compileSilently: each classified: #autogenerated]! !

!ArduinoPlayerTranslator methodsFor: 'compiling' stamp: 'Richo 12/27/2011 18:11'!
createTempProgramSubclassInstVarNames: aString instVarTypes: anArray 
	SystemChangeNotifier uniqueInstance doSilently: [
		ArduinoSketch subclass: self programName
			instanceVariableNames: aString
			instanceVariableTypes: anArray
			classVariableNames: ''
			poolDictionaries: ''
			category: 'ArduinoPrograms']! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/15/2011 16:53'!
currentPlayer
	^ currentMorph assuredPlayer! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/15/2011 16:53'!
currentPlayerClass
	^ self currentPlayer class! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/16/2011 12:24'!
delayOfScript: scriptName
 	"To be in accordance with the script tick rate, we put a delay at the end of each task. How long the delay would be?"
	^ (1000 / (self currentPlayer scriptInstantiationForSelector: scriptName) tickingRate) rounded asString! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/16/2011 13:07'!
etoysMessages
^ {
"#grouped actually doesn't do anything so we just take it out"
#grouped -> [:node | node become: (node receiver parent: node parent)].

"I don't know why but recent versions of Etoys use this weird messages instead of the comparison operators:"
#eToysLT: -> [:node | node selector: #<].
#eToysLE: -> [:node | node selector: #<=].
#eToysEQ: -> [:node | node selector: #=].
#eToysNE: -> [:node | node selector: #~=].
#eToysGT: -> [:node | node selector: #>].
#eToysGE: -> [:node | node selector: #>=].
} as: Dictionary! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/16/2011 12:24'!
frequencyOfScript: scriptName
	"To be in accordance with the script frequency property we enclose the script call in a #timesRepeat: message. How many times should we repeat the script?"
	^ (self currentPlayer scriptInstantiationForSelector: scriptName) frequency rounded asString! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 10/21/2011 19:15'!
hashString
	^ self hash asString! !

!ArduinoPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 10/21/2011 16:33'!
initialize
	super initialize.
	self collectRelevantInformation! !

!ArduinoPlayerTranslator methodsFor: 'initializing' stamp: 'Richo 8/11/2011 17:23'!
initializeWith: anArduinoMorph
	"By default we set the brick as the currentMorph"
	mainMorph := currentMorph := anArduinoMorph.
	self initialize! !

!ArduinoPlayerTranslator methodsFor: 'testing' stamp: 'Richo 8/16/2011 13:10'!
isValidReceiver: aParseNode 
	"If the parse node is a variable it *must* be one of the valid morphs or an argument variable. I'm not sure about this, actually so we might have to test it more"
	| res |
	aParseNode isVariable
		ifTrue: [res := (aParseNode methodNode allArgumentVariables
							anySatisfy: [:each | each = aParseNode name])
						or: [(self morphFor: aParseNode name) notNil].
			res
				ifFalse: [self halt].
			^ res].
	^ true! !

!ArduinoPlayerTranslator methodsFor: 'testing' stamp: 'Richo 8/16/2011 13:05'!
isValidSelector: selector 
	"Checks if the selector is one of the following:
	* An arduino primitive rule.
	* A player translator rule
	* An etoys message.
	* A slot accessor.
	* A script selector."
	^ self class allPrimitiveRulesSelectors asSet , self rules keys , self etoysMessages keys , self allGetterSelectors , self allSetterSelectors , self allScriptSelectors includes: selector! !

!ArduinoPlayerTranslator methodsFor: 'creating parse nodes' stamp: 'Richo 8/16/2011 13:09'!
literal: aLiteralObject
	^ RBLiteralValueNode value: aLiteralObject! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/16/2011 13:15'!
morphFor: receiverName
	"Returns the morph represented by the argument or nil if it's not a valid morph"
	^ receiverName = 'self'
		ifTrue: [currentMorph]
		ifFalse: [self allMorphs
				detect: [:each | each notNil and: [each assuredPlayer uniqueNameForReference = receiverName]] ifNone: [nil]]! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/16/2011 12:25'!
programClass
	"Returns the temporary class we are using to compile the code"
	^ Smalltalk at: self programName ifAbsent: []! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 8/16/2011 12:25'!
programName
	^ #ArduinoEtoysProgram! !

!ArduinoPlayerTranslator methodsFor: 'creating parse nodes' stamp: 'Richo 10/21/2011 17:11'!
receiver: receiverNode selector: selectorSymbol arguments: anArray
	^ RBMessageNode receiver: receiverNode selector: selectorSymbol arguments: anArray! !

!ArduinoPlayerTranslator methodsFor: 'compiling' stamp: 'Richo 8/16/2011 12:25'!
removeTempProgramSubclass
	"After translation is done we silently remove the temporary class"
	SystemChangeNotifier uniqueInstance doSilently: [
		self programClass ifNotNilDo: [:class | class removeFromSystem]]! !

!ArduinoPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 10/24/2011 14:32'!
rules
"These are the rules to modify the parse tree of the ArduinoPlayer scripts. Most of the time, I just need to rename the selector, change the receiver to self and set the correct list of arguments. Other times the rule is a little more complicated"
^ (
	self commonDevicesRules,
	self scriptingRules
	) as: Dictionary! !

!ArduinoPlayerTranslator methodsFor: 'iterating' stamp: 'Richo 8/15/2011 16:49'!
scriptSelectorsDo: aBlock 
	"Iterates over the scriptSelectors dictionary taking care of setting the current morph"
	scriptSelectors
		keysAndValuesDo: [:morph :selectors | self
				withCurrentMorph: morph
				do: [selectors do: aBlock]] ! !

!ArduinoPlayerTranslator methodsFor: 'accessing-rules' stamp: 'Richo 10/24/2011 14:35'!
scriptingRules
| selfVar |
selfVar := self variable: 'self'.
^ {
#startScript: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [	
		node arguments: {self literal: (self asLegalScriptFlag: node arguments first value), ' = true;'};
			 selector: #c:;
			 receiver: selfVar].
	super acceptMessageNode: node].

#stopScript: -> [:node |
	self withCurrentMorph: (self morphFor: node receiver name) do: [	
		node arguments: {self literal: (self asLegalScriptFlag: node arguments first value), ' = false;'};
			 selector: #c:;
			 receiver: selfVar].
	super acceptMessageNode: node].

#pauseScript: -> [:node | "Do the same as #stopScript:"
	(self rules at: #stopScript:) value: node].

#doScript: -> [:node | | args | args := node arguments. "This is the same as calling the script directly"
	node arguments: #(); selector: args first value.
	self visitNode: node "We need to visit the node again to fix the selector and the receiver"]
}! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 12/27/2011 16:50'!
slotForGetter: getterSelector
	"Returns the slot accessed by these selector"
	| slotName |
	slotName := currentMorph assuredPlayer slotInfo keys
				detect: [:slot | (Utilities getterSelectorFor: slot)
						= getterSelector].
	^ self asLegalName: slotName! !

!ArduinoPlayerTranslator methodsFor: 'accessing' stamp: 'Richo 12/27/2011 16:50'!
slotForSetter: setterSelector
	"Returns the slot accessed by these selector"
	| slotName |
	slotName := currentMorph assuredPlayer slotInfo keys
				detect: [:slot | (Utilities setterSelectorFor: slot)
						= setterSelector].
	^ self asLegalName: slotName! !

!ArduinoPlayerTranslator methodsFor: 'translating' stamp: 'Richo 10/24/2011 14:17'!
translateMainScript: scriptName of: scriptOwner
	"It translates the script to arduino c. It creates a temporary ArduinoCProgram subclass to hold the code which will be translated to c. After translation is done, it removes the temporary class.
	Steps:
	* Collect relevant information (motors and sensors composing the robot, slots, accessors and scripts).
	* Compile #setUp method.
	* Compile #main method.
	* Compile the script task.
	* Compile all scripts.
	* Remove temporary class.	
	* Compile temporary class.
	* Translate program to nxc
	* Return nxc program"
	self collectRelevantInformation.
	self
		assert: [self allMorphs includes: scriptOwner]
		description: 'The morph is not connected to the Arduino board';
		assert: [scriptOwner assuredPlayer class scripts includesKey: scriptName]
		description: 'The script doesn''t belong to the morph'.
	methods := Set new. 
	self
		scriptSelectorsDo: [:each | 
			| node | 
			self visitNode: (node := self currentPlayerClass parseTreeFor: each).
			(variables at: currentMorph)
				at: (self asScriptFlag: each)
				put: #Boolean -> (each = scriptName and: [currentMorph = scriptOwner]).
			(variables at: currentMorph) at: (self asScriptLastTime: each) put: #CUnsignedLong -> 0.
			methods add: node formattedCode].
	self
		withCurrentMorph: scriptOwner
		do: [self compileLoopMethod; compileSetUpMethod].
	self removeTempProgramSubclass;
		compileTempProgramSubclass.
	^ self programClass translatedToArduinoC: #main named: scriptName! !

!ArduinoPlayerTranslator methodsFor: 'creating parse nodes' stamp: 'Richo 8/16/2011 13:09'!
variable: variableName
	^ RBVariableNode named: variableName! !

!ArduinoPlayerTranslator methodsFor: 'iterating' stamp: 'Richo 8/16/2011 12:27'!
variablesDo: aBlock 
	"Iterates over the variables dictionary taking care of setting the current morph"
	variables
		keysAndValuesDo: [:morph :vars | self
				withCurrentMorph: morph
				do: [vars keysAndValuesDo: aBlock]] ! !

!ArduinoPlayerTranslator methodsFor: 'translating' stamp: 'Richo 8/15/2011 16:49'!
withCurrentMorph: aMorph do: aBlock
	"It executes the block using the morph argument as our current morph. It also remembers the old morph, so it's safe to nest this expression.
	The current morph is used whenever we need a legal name for a variable/method name. It defines which morph is currently represented by the 'self' variable"
	| old |
	old := currentMorph.
	currentMorph := aMorph.
	aBlock value.
	currentMorph := old! !

!ArduinoMorph methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 8/11/2011 16:05'!
detectMainMorph
	^ self! !

!ArduinoMorph methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 10/24/2011 12:50'!
download: anArduinoCTranslatedProgram
	anArduinoCTranslatedProgram 
		uploadOnPort: portName 
		arduinoType: (ArduinoType withFriendlyName: boardType)! !

!ArduinoMorph methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 8/11/2011 16:08'!
playerTranslator
	^ ArduinoPlayerTranslator on: self! !

ArduinoProtocol subclass: #ArduinoCompiledMode
	instanceVariableNames: 'isConnected'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Arduino-EtoysCompiler'!

!ArduinoCompiledMode methodsFor: 'analog pins' stamp: 'Richo 12/23/2011 16:37'!
activateAnalogPin: pinNumber
"Do nothing"! !

!ArduinoCompiledMode methodsFor: 'digital pins' stamp: 'Richo 12/23/2011 16:38'!
activateDigitalPort: portNumber
	"Do nothing"! !

!ArduinoCompiledMode methodsFor: 'analog pins' stamp: 'Richo 12/23/2011 16:37'!
analogRead: pinNumber
	"Do nothing"
	^ 0! !

!ArduinoCompiledMode methodsFor: 'analog pins' stamp: 'Richo 12/27/2011 15:31'!
analogWrite: pin value: value
	"Do nothing"! !

!ArduinoCompiledMode methodsFor: 'servos' stamp: 'Richo 12/23/2011 16:39'!
attachServoToPin: pinNumber
	"Do nothing"! !

!ArduinoCompiledMode methodsFor: 'connection' stamp: 'Richo 12/23/2011 16:39'!
connectOnPort: portName boardType: anArduinoType
	isConnected := true! !

!ArduinoCompiledMode methodsFor: 'analog pins' stamp: 'Richo 12/23/2011 16:37'!
deactivateAnalogPin: pinNumber
	"Do nothing"! !

!ArduinoCompiledMode methodsFor: 'digital pins' stamp: 'Richo 12/23/2011 16:38'!
deactivateDigitalPort: portNumber
	"Do nothing"! !

!ArduinoCompiledMode methodsFor: 'servos' stamp: 'Richo 12/23/2011 16:39'!
detachServoFromPin: pinNumber
	"Do nothing"! !

!ArduinoCompiledMode methodsFor: 'digital pins' stamp: 'Richo 12/23/2011 16:38'!
digitalPin: pinNumber mode: mode
	"Do nothing"! !

!ArduinoCompiledMode methodsFor: 'digital pins' stamp: 'Richo 12/23/2011 16:38'!
digitalRead: pinNumber
	"Do nothing"
	^ 0! !

!ArduinoCompiledMode methodsFor: 'digital pins' stamp: 'Richo 12/23/2011 16:39'!
digitalWrite: pinNumber value: value
	"Do nothing"! !

!ArduinoCompiledMode methodsFor: 'connection' stamp: 'Richo 12/23/2011 16:39'!
disconnect
	isConnected := false! !

!ArduinoCompiledMode methodsFor: 'testing' stamp: 'Richo 12/23/2011 15:11'!
isCompiled
	^ true! !

!ArduinoCompiledMode methodsFor: 'connection' stamp: 'Richo 12/23/2011 16:45'!
isConnected
	^ isConnected ifNil: [false]! !

!ArduinoCompiledMode methodsFor: 'accessing' stamp: 'Richo 12/23/2011 16:29'!
nameSymbol
	^ #compiled! !

!ArduinoCompiledMode methodsFor: 'servos' stamp: 'Richo 12/23/2011 16:39'!
servoOnPin: pinNumber angle: degrees
	"Do nothing"! !

!ArduinoCompiledMode methodsFor: 'pin modes' stamp: 'Richo 12/23/2011 16:40'!
valueForInputMode
	"Do nothing"
	^ 0! !

!ArduinoCompiledMode methodsFor: 'pin modes' stamp: 'Richo 12/23/2011 16:40'!
valueForOutputMode
	"Do nothing"
	^ 0! !

!ArduinoCompiledMode methodsFor: 'pin modes' stamp: 'Richo 12/23/2011 16:40'!
valueForPwmMode
	"Do nothing"
	^ 0! !

!ArduinoCompiledMode methodsFor: 'pin modes' stamp: 'Richo 12/23/2011 16:40'!
valueForServoMode
	"Do nothing"
	^ 0! !

!ArduinoPlayer methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 12/23/2011 16:16'!
getArduinoMode
	^ self costume renderedMorph mode nameSymbol! !

!ArduinoPlayer methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 8/11/2011 16:08'!
isUsingCompiledMode
	"An error might be signaled if this message is sent to an unplugged device"
	^ [self costume renderedMorph detectMainMorph isUsingCompiledMode] on: Error do: [false]! !

!ArduinoPlayer methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 8/11/2011 16:09'!
newScriptorAround: aPhrase
	| scriptEditor |
	Transcript show: 'newScriptorAround:'; cr.
	^ scriptEditor := super newScriptorAround: aPhrase.
	"^ NxtScriptEditorMorph fromScriptEditor: scriptEditor phrase: aPhrase"! !

!ArduinoPlayer methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 8/11/2011 16:09'!
scriptEditorFor: aSelector
	Transcript show: 'scriptEditorFor:'; cr.
	^ super scriptEditorFor: aSelector! !

!ArduinoPlayer methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 8/11/2011 16:09'!
scriptEditorForNoCng: aSelector
	Transcript show: 'scriptEditorForNoCng:'; cr.
	^ super scriptEditorForNoCng: aSelector.! !

!ArduinoPlayer methodsFor: '*arduino-etoyscompiler' stamp: 'Richo 12/23/2011 16:17'!
setArduinoMode: aSymbol
	self getArduinoMode = aSymbol ifTrue: [^self].
	self costume renderedMorph
		mode: (aSymbol caseOf: {
			[#firmata] -> [Firmata new].
			[#direct] -> [ArduinoDirectMode new].
			[#compiled] -> [ArduinoCompiledMode new].
			} otherwise: [^self]).

	"Horrible hack below"
	World allMorphsDo: [:morph | | viewer |
		"Update scripts category of all opened viewers"
		((morph isKindOf: StandardViewer) and: [morph playerScripted isKindOf: self class officialClass]) 
			ifTrue: [morph updateScriptsCategory].
		"Also update closed viewers"
		((morph isKindOf: ViewerFlapTab) and: [(viewer := morph referent findA: StandardViewer) notNil])
			ifTrue: [viewer updateScriptsCategory].
		"Finally, update all opened script editors"
		((morph isKindOf: ScriptEditorMorph) and: [morph playerScripted isKindOf: self class officialClass]) 
			ifTrue: [morph replaceRow1; minWidth: nil]]! !
